@@ -1,6 +1,8 @@
 import argparse
+import sys
 
 import whack.args
+from whack.errors import WhackUserError
 
 env_default = whack.args.env_default(prefix="WHACK")
 
@@ -12,7 +14,11 @@ def main(argv, create_operations):
         indices=args.indices,
         enable_build=args.enable_build,
     )
-    args.func(operations, args)
+    try:
+        args.func(operations, args)
+    except WhackUserError as error:
+        sys.stderr.write("{0}: {1}\n".format(type(error).__name__, error.message))
+        exit(1)
 
 
 def parse_args(argv):
@@ -21,7 +27,7 @@ def parse_args(argv):
         InstallCommand("get-package"),
         DeployCommand(),
         CreateSourceTarballCommand(),
-        BuildPackageTarballCommand(),
+        GetPackageTarballCommand(),
     ]
     
     parser = argparse.ArgumentParser()
@@ -62,7 +68,7 @@ class InstallCommand(object):
         self.name = name
     
     def create_parser(self, subparser):
-        subparser.add_argument('package')
+        subparser.add_argument('package_source', metavar="package-source")
         subparser.add_argument('target_dir', metavar="target-dir")
         _add_build_params_args(subparser)
     
@@ -74,7 +80,7 @@ class InstallCommand(object):
         else:
             raise Exception("Unrecognised operation")
             
-        operation(args.package, args.target_dir, params=args.params)
+        operation(args.package_source, args.target_dir, params=args.params)
 
 
 class DeployCommand(object):
@@ -107,7 +113,7 @@ class CreateSourceTarballCommand(object):
         print source_tarball.path
 
 
-class BuildPackageTarballCommand(object):
+class GetPackageTarballCommand(object):
     name = "get-package-tarball"
     
     def create_parser(self, subparser):
