@@ -1,24 +1,21 @@
 import os
 
-from catchy import xdg_directory_cacher, NoCachingStrategy
-
 from .sources import PackageSourceFetcher, create_source_tarball
 from .providers import create_package_provider
 from .deployer import PackageDeployer
 from .tempdir import create_temporary_dir
 from .files import read_file
 from .tarballs import create_tarball
+from .packagerequests import PackageRequest
+from .caching import create_cacher_factory
 
 
 def create(caching_enabled, indices=None, enable_build=True):
-    if not caching_enabled:
-        cacher = NoCachingStrategy()
-    else:
-        cacher = xdg_directory_cacher("whack/builds")
+    cacher_factory = create_cacher_factory(caching_enabled=caching_enabled)
     
     package_source_fetcher = PackageSourceFetcher(indices)
     package_provider = create_package_provider(
-        cacher,
+        cacher_factory,
         enable_build=enable_build,
         indices=indices,
     )
@@ -33,20 +30,14 @@ class Operations(object):
         self._package_provider = package_provider
         self._deployer = deployer
         
-    def install(self, package_name, install_dir, params=None):
-        self.get_package(package_name, install_dir, params)
+    def install(self, source_name, install_dir, params=None):
+        self.get_package(source_name, install_dir, params)
         self.deploy(install_dir)
         
-    def get_package(self, package_name, install_dir, params=None):
-        if params is None:
-            params = {}
-            
-        with self._package_source_fetcher.fetch(package_name) as package_source:    
-            self._package_provider.provide_package(
-                package_source,
-                params,
-                install_dir
-            )
+    def get_package(self, source_name, install_dir, params=None):
+        with self._package_source_fetcher.fetch(source_name) as package_source:
+            request = PackageRequest(package_source, params)
+            self._package_provider.provide_package(request, install_dir)
         
     def deploy(self, package_dir, target_dir=None):
         return self._deployer.deploy(package_dir, target_dir)
