@@ -3,25 +3,27 @@ import json
 import tempfile
 import uuid
 import re
+import errno
 
 import blah
 
 from .hashes import Hasher
-from .files import copy_dir, mkdir_p, copy_file, delete_dir
+from .files import copy_dir, copy_file, delete_dir
 from .tarballs import extract_tarball, create_tarball
 from .indices import read_index
+from .errors import FileNotFoundError, WhackUserError
 
 
 _whack_source_uri_suffix = ".whack-source"
 
 
-class PackageSourceNotFound(Exception):
-    def __init__(self, package_name):
-        message = "Could not find source for package: {0}".format(package_name)
+class PackageSourceNotFound(WhackUserError):
+    def __init__(self, source_name):
+        message = "Could not find package source: {0}".format(source_name)
         Exception.__init__(self, message)
         
         
-class SourceHashMismatch(Exception):
+class SourceHashMismatch(WhackUserError):
     def __init__(self, expected_hash, actual_hash):
         message = "Expected hash {0} but was {1}".format(
             expected_hash,
@@ -37,160 +39,109 @@ class PackageSourceFetcher(object):
         else:
             self._indices = indices
     
-    def fetch(self, package, lazy=True):
-        fetchers = [IndexFetcher(self._indices), SourceControlFetcher()]
-        if lazy:
-            fetchers.append(WhackSourceUriFetcher())
-        fetchers += [
+    def fetch(self, source_name):
+        index_fetchers = map(IndexFetcher, self._indices)
+        fetchers = index_fetchers + [
+            SourceControlFetcher(),
             HttpFetcher(),
             LocalPathFetcher(),
         ]
         for fetcher in fetchers:
-            package_source = self._fetch_with_fetcher(fetcher, package)
+            package_source = self._fetch_with_fetcher(fetcher, source_name)
             if package_source is not None:
-                return package_source
-                
-        raise PackageSourceNotFound(package)
+                try:
+                    self._verify(source_name, package_source)
+                    return package_source
+                except:
+                    package_source.__exit__()
+                    raise
+        raise PackageSourceNotFound(source_name)
         
-    def _fetch_with_fetcher(self, fetcher, package):
-        if fetcher.can_fetch(package):
-            return fetcher.fetch(package)
+    def _fetch_with_fetcher(self, fetcher, source_name):
+        if fetcher.can_fetch(source_name):
+            return fetcher.fetch(source_name)
         else:
             return None
+            
+    def _verify(self, source_name, package_source):
+        whack_source_uri_regex = \
+            r"(?:^|/|-)([^./-]*){0}$".format(re.escape(_whack_source_uri_suffix))
+        result = re.search(whack_source_uri_regex, source_name)
+        if result:
+            expected_hash = result.group(1)
+            actual_hash = package_source.source_hash()
+            if expected_hash != actual_hash:
+                raise SourceHashMismatch(expected_hash, actual_hash)
 
 
 class IndexFetcher(object):
-    def __init__(self, indices):
-        self._indices = indices
+    def __init__(self, index_uri):
+        self._index_uri = index_uri
     
-    def can_fetch(self, package):
-        return re.match(r"^[a-z0-9\-]+$", package)
+    def can_fetch(self, source_name):
+        return re.match(r"^[a-z0-9\-]+$", source_name)
         
-    def fetch(self, package_name):
-        for index_uri in self._indices:
-            source = self._fetch_from_index(index_uri, package_name)
-            if source is not None:
-                return source
-        return None
-        
-    def _fetch_from_index(self, index_uri, package_name):
-        index = read_index(index_uri)
-        package_source_filename = package_name + _whack_source_uri_suffix
+    def fetch(self, source_name):
+        index = read_index(self._index_uri)
+        package_source_filename = source_name + _whack_source_uri_suffix
         package_source_entry = index.find_by_name(package_source_filename)
         if package_source_entry is None:
             return None
         else:
-            return WhackSourceUriFetcher().fetch(package_source_entry.url)
+            return HttpFetcher().fetch(package_source_entry.url)
     
 
 class SourceControlFetcher(object):
-    def can_fetch(self, package):
-        return blah.is_source_control_uri(package)
+    def can_fetch(self, source_name):
+        return blah.is_source_control_uri(source_name)
         
-    def fetch(self, source_control_uri):
+    def fetch(self, source_name):
         def fetch_archive(destination_dir):
-            blah.archive(source_control_uri, destination_dir)
-            return destination_dir
+            blah.archive(source_name, destination_dir)
         
-        return _create_temporary_package_source(fetch_archive)
+        return _create_temporary_package_source(source_name, fetch_archive)
         
         
 class LocalPathFetcher(object):
-    def can_fetch(self, package):
+    def can_fetch(self, source_name):
         return (
-            package.startswith("/") or
-            package.startswith("./") or
-            package.startswith("../") or 
-            package == "." or
-            package == ".."
+            source_name.startswith("/") or
+            source_name.startswith("./") or
+            source_name.startswith("../") or 
+            source_name == "." or
+            source_name == ".."
         )
         
-    def fetch(self, path):
-        if os.path.isfile(path):
-            return self._fetch_package_from_tarball(path)
+    def fetch(self, source_name):
+        if os.path.isfile(source_name):
+            return self._fetch_package_from_tarball(source_name)
         else:
-            return PackageSource(path)
+            return PackageSource.local(source_name)
     
     def _fetch_package_from_tarball(self, tarball_path):
         def fetch_directory(destination_dir):
             extract_tarball(tarball_path, destination_dir, strip_components=1)
             return destination_dir
         
-        return _create_temporary_package_source(fetch_directory)
+        return _create_temporary_package_source(tarball_path, fetch_directory)
         
 
 class HttpFetcher(object):
-    def can_fetch(self, package):
-        return package.startswith("http://")
+    def can_fetch(self, source_name):
+        return source_name.startswith("http://")
         
-    def fetch(self, url):
+    def fetch(self, source_name):
         def fetch_directory(temp_dir):
-            extract_tarball(url, temp_dir, strip_components=1)
-            return temp_dir
+            extract_tarball(source_name, temp_dir, strip_components=1)
             
-        return _create_temporary_package_source(fetch_directory)
+        return _create_temporary_package_source(source_name, fetch_directory)
 
 
-class WhackSourceUriFetcher(object):
-    def can_fetch(self, package):
-        return package.endswith(_whack_source_uri_suffix)
-        
-    def fetch(self, uri):
-        result = re.search(r"/(?:([^./]*)-)?([^./]*)\..*$", uri)
-        name = result.group(1)
-        source_hash = result.group(2)
-        return RemotePackageSource(name, source_hash, uri)
-
-
-class RemotePackageSource(object):
-    def __init__(self, name, source_hash, uri):
-        self._name = name
-        self._source_hash = source_hash
-        self._uri = uri
-        self._package_source_dir = None
-    
-    def name(self):
-        return self._name
-    
-    def source_hash(self):
-        return self._source_hash
-    
-    def write_to(self, target_dir):
-        self._fetch_package_source().write_to(target_dir)
-    
-    def _fetch_package_source(self):
-        if self._package_source_dir is None:
-            self._package_source_dir = _temporary_path()
-            mkdir_p(self._package_source_dir)
-            fetcher = PackageSourceFetcher()
-            with fetcher.fetch(self._uri, lazy=False) as package_source:
-                self._verify_hash(package_source)
-                
-                package_source.write_to(self._package_source_dir)
-            
-        return PackageSource(self._package_source_dir)
-    
-    def _verify_hash(self, package_source):
-        expected_hash = self._source_hash
-        actual_hash = package_source.source_hash()
-        if expected_hash != actual_hash:
-            raise SourceHashMismatch(expected_hash, actual_hash)
-    
-    def __enter__(self):
-        return self
-        
-    def __exit__(self, *args):
-        if self._package_source_dir is not None:
-            delete_dir(self._package_source_dir)
-
-
-def _create_temporary_package_source(fetch_package_source_dir):
+def _create_temporary_package_source(uri, fetch_package_source_dir):
     temp_dir = _temporary_path()
     try:
-        return TemporaryPackageSource(
-            fetch_package_source_dir(temp_dir),
-            temp_dir
-        )
+        fetch_package_source_dir(temp_dir)
+        return PackageSource(temp_dir, uri, is_temp=True)
     except:
         delete_dir(temp_dir)
         raise
@@ -205,13 +156,19 @@ def _is_tarball(path):
 
 
 class PackageSource(object):
-    def __init__(self, path):
+    @staticmethod
+    def local(path):
+        return PackageSource(path, path, is_temp=False)
+    
+    def __init__(self, path, uri, is_temp):
         self._path = path
+        self.uri = uri
         self._description = _read_package_description(path)
+        self._is_temp = is_temp
     
     def name(self):
         return self._description.name()
-        
+    
     def full_name(self):
         name = self.name()
         source_hash = self.source_hash()
@@ -230,7 +187,19 @@ class PackageSource(object):
     def write_to(self, target_dir):
         for source_dir in self._source_paths():
             target_sub_dir = os.path.join(target_dir, source_dir)
-            _copy_dir_or_file(os.path.join(self._path, source_dir), target_sub_dir)
+            try:
+                _copy_dir_or_file(
+                    os.path.join(self._path, source_dir),
+                    target_sub_dir
+                )
+            except IOError as error:
+                if error.errno == errno.ENOENT:
+                    raise FileNotFoundError()
+                else:
+                    raise error
+    
+    def description(self):
+        return self._description
     
     def _source_paths(self):
         return ["whack"] + self._description.source_paths()
@@ -239,7 +208,8 @@ class PackageSource(object):
         return self
         
     def __exit__(self, *args):
-        pass
+        if self._is_temp:
+            delete_dir(self._path)
 
 
 def _copy_dir_or_file(source, destination):
@@ -247,18 +217,6 @@ def _copy_dir_or_file(source, destination):
         copy_dir(source, destination)
     else:
         copy_file(source, destination)
-
-
-class TemporaryPackageSource(object):
-    def __init__(self, path, temp_dir):
-        self._path = path
-        self._temp_dir = temp_dir
-    
-    def __enter__(self):
-        return PackageSource(self._path)
-    
-    def __exit__(self, *args):
-        delete_dir(self._temp_dir)
         
 
 def _read_package_description(package_src_dir):
@@ -278,12 +236,18 @@ class DictBackedPackageDescription(object):
     def name(self):
         return self._values.get("name", None)
         
+    def param_slug(self):
+        return self._values.get("paramSlug", None)
+        
     def source_paths(self):
         return self._values.get("sourcePaths", [])
+        
+    def default_params(self):
+        return self._values.get("defaultParams", {})
 
 
 def create_source_tarball(source_dir, tarball_dir):
-    package_source = PackageSource(source_dir)
+    package_source = PackageSource.local(source_dir)
     full_name = package_source.full_name()
     filename = "{0}{1}".format(full_name, _whack_source_uri_suffix)
     path = os.path.join(tarball_dir, filename)
